! $Id: MathCodeSheep.f90,v 1.9 2007/09/17 12:51:21 vaden Exp $

! This file implements functions in Fortran90 that correspondes
! to the Mathematica functions Drop[], Append[], Prepend[], Join[],
! Outer[], Extract[] and Equal[]. This is done via generic interfaces, so that
! they can be called with either integer or real vectors. Just include
! MathCodeSheep, and you are ready to use them.

! Generic function             Corresponding Mathematica expr.
! -------------------------------------------------------------
! mc_drop_edge(vec, n)         Drop[vec, n]
! mc_drop_middle(vec, n, m)    Drop[vec, {n, m}]
! mc_append(vec, elem)         Append[vec, elem]
! mc_prepend(vec, elem)        Prepend[vec, elem]
! mc_join(vec1, vec2)          Join[vec1, vec2]
! mc_outer_product(vec1, vec2) Outer[Times, vec1, vec2]
! mc_extract_N0(arg, sel)      Extract[] returning element
! mc_extract_N1(arg, sel)      Extract[] returning vector
! mc_extract_N2(arg, sel)      Extract[] returning matrix
! mc_extract_N3(arg, sel)      Extract[] returning rank 3 tensor

! Drop[{...}, n]
!   drop the first n (or last n if n is negative) elements from vector
!
! mc_equal(arg1,arg2)          Equal[arg1, arg2], where arg1 and arg2 are arrays 
! ------------------------------------------------------------------------------



! -------------------------------------------------------------
!  Module containing generic interfaces                      |

module MathCodeSheep

interface mc_drop_edge
	module procedure  mc_drop_edge_i,mc_drop_edge_r
end interface ! mc_drop_edge

interface mc_drop_middle
	module procedure  mc_drop_middle_i,mc_drop_middle_r
end interface ! mc_drop_middle

interface mc_append
  module procedure  mc_append_i,mc_append_r
end interface ! mc_append

interface mc_prepend
  module procedure mc_prepend_i,mc_prepend_r
end interface ! mc_prepend

interface mc_join
 module procedure mc_join_i,mc_join_r
 end interface ! mc_join

interface mc_outer_product
  module procedure  mc_outer_product_i,mc_outer_product_r
end interface ! mc_outer_product

interface mc_extract_N0
  module procedure mc_extract_10_i,mc_extract_20_i,mc_extract_30_i,mc_extract_40_i,&
  mc_extract_10_r,mc_extract_20_r,mc_extract_30_r,mc_extract_40_r
end interface ! mc_extract_N0

interface mc_extract_N1
 module procedure  mc_extract_21_i,mc_extract_31_i,mc_extract_41_i,&
 mc_extract_21_r,mc_extract_31_r,mc_extract_41_r
end interface ! mc_extract_N1

interface mc_extract_N2
 module procedure mc_extract_32_i,mc_extract_42_i,mc_extract_32_r,mc_extract_42_r
end interface ! mc_extract_N2

interface mc_extract_N3
 module procedure mc_extract_43_i,mc_extract_43_r
end interface ! mc_extract_N3

interface mc_equal
 module procedure  mc_equal_1_i,mc_equal_1d_i,mc_equal_1_r,mc_equal_1d_r,&
 mc_equal_2_i,mc_equal_2d_i,mc_equal_2_r,mc_equal_2d_r,&
 mc_equal_3_i,mc_equal_3d_i,mc_equal_3_r,mc_equal_3d_r,&
 mc_equal_4_i,mc_equal_4_r
end interface ! mc_equal

interface mc_crossprod_2   
 module procedure mc_crossprod_2_i,mc_crossprod_2_r
end interface ! mc_crossprod_2

interface mc_crossprod_3   
 module procedure mc_crossprod_3_i,mc_crossprod_3_r
end interface ! mc_crossprod_3

interface mc_crossprod_4   
 module procedure mc_crossprod_4_i,mc_crossprod_4_r
end interface ! mc_crossprod_4

interface mc_mkc
 module procedure mc_mkc_1_i,mc_mkc_2_i,mc_mkc_3_i,mc_mkc_4_i,&
 mc_mkc_1_r,mc_mkc_2_r,mc_mkc_3_r,mc_mkc_4_r
end interface ! mc_mck

interface mc_mkpc
 module procedure mc_mkpc_1_i,mc_mkpc_2_i,mc_mkpc_3_i,mc_mkpc_4_i,&
 mc_mkpc_1_r,mc_mkpc_2_r,mc_mkpc_3_r,mc_mkpc_4_r
end interface ! mc_mkpc

interface mc_getpart1
 module procedure mc_getpart1_1_i,mc_getpart1_1_r,mc_getpart1_2_i,mc_getpart1_2_r,&
 mc_getpart1_3_i,mc_getpart1_3_r,mc_getpart1_4_i,mc_getpart1_4_r
end interface ! mc_getpart1

interface mc_getpart2
 module procedure mc_getpart2_2_i,mc_getpart2_2_r,mc_getpart2_3_i,mc_getpart2_3_r,&
 mc_getpart2_4_i,mc_getpart2_4_r
end interface ! mc_getpart2

interface mc_getpart3
 module procedure mc_getpart3_3_i,mc_getpart3_3_r,mc_getpart3_4_i,mc_getpart3_4_r
end interface ! mc_getpart3

interface mc_getpart4
 module procedure mc_getpart4_4_i,mc_getpart4_4_r
end interface ! mc_getpart4

contains




function mc_drop_edge_i(invector, n) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: n
	integer(mc_int) outvector(size(invector,1)-abs(n))

	if (n > 0) then
		outvector = invector(n+1:)
	else
		outvector = invector(:ubound(invector,1)-n-1)
	end if
end function mc_drop_edge_i

function mc_drop_edge_r(invector, n) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: n
	real(mc_real) outvector(size(invector,1)-abs(n))

	if (n > 0) then
		outvector = invector(n+1:)
	else
		outvector = invector(:ubound(invector,1)-n-1)
	end if
end function mc_drop_edge_r

! Drop[{...}, {n, m}]
!   drop elements n through m from vector
! -------------------------------------------------------------

function mc_drop_middle_i(invector, start, end) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: start, end
	integer(mc_int) outvector(size(invector,1)-(end-start+1))

	outvector(1:start-1) = invector(1:start-1)
	outvector(start:) = invector(end+1:)
end function mc_drop_middle_i

function mc_drop_middle_r(invector, start, end) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: start, end
	real(mc_real) outvector(size(invector,1)-(end-start+1))

	outvector(1:start-1) = invector(1:start-1)
	outvector(start:) = invector(end+1:)
end function mc_drop_middle_r

! Append[{...}, n]
!   append element to vector
! -------------------------------------------------------------

function mc_append_i(invector, element) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: element
	integer(mc_int) outvector(size(invector,1)+1)

	outvector(1:size(invector,1)) = invector
	outvector(size(invector,1)+1) = element
end function mc_append_i

function mc_append_r(invector, element) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector(:)
	real(mc_real), intent(in) :: element
	real(mc_real) outvector(size(invector,1)+1)

	outvector(1:size(invector,1)) = invector
	outvector(size(invector,1)+1) = element
end function mc_append_r

! Prepend[{...}, n]
!   prepend element to vector
! -------------------------------------------------------------

function mc_prepend_i(invector, element) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector(:)
	integer(mc_int), intent(in) :: element
	integer(mc_int) outvector(size(invector,1)+1)

	outvector(1) = element
	outvector(2:size(invector,1)+1) = invector
end function mc_prepend_i

function mc_prepend_r(invector, element) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector(:)
	real(mc_real), intent(in) :: element
	real(mc_real) outvector(size(invector,1)+1)

	outvector(1) = element
	outvector(2:size(invector,1)+1) = invector
end function mc_prepend_r

! Join[{...}, {...}]
!   concatenate two vectors
! -------------------------------------------------------------

function mc_join_i(invector1, invector2) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector1(:)
	integer(mc_int), intent(in) :: invector2(:)
	integer(mc_int) outvector(size(invector1,1)+size(invector2,1))

	outvector(1:size(invector1,1)) = invector1
	outvector(size(invector1,1)+1:) = invector2
end function mc_join_i

function mc_join_r(invector1, invector2) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector1(:)
	real(mc_real), intent(in) :: invector2(:)
	real(mc_real) outvector(size(invector1,1)+size(invector2,1))

	outvector(1:size(invector1,1)) = invector1
	outvector(size(invector1,1)+1:) = invector2
end function mc_join_r

! Outer[Times, {...}, {...}]
!   outer product of two vectors
! -------------------------------------------------------------

function mc_outer_product_i(invector1, invector2) result (outvector)
	use MathCodePrecision
	integer(mc_int), intent(in) :: invector1(:), invector2(:)
	integer(mc_int) outvector(size(invector2,1), size(invector1,1))
        integer(mc_int) i,j 
        do i=1,size(invector1,1)
        	do j=1,size(invector2,1)
			outvector(j,i)= invector1(i)* invector2(j)
		end do
	end do

end function mc_outer_product_i

function mc_outer_product_r(invector1, invector2) result (outvector)
	use MathCodePrecision
	real(mc_real), intent(in) :: invector1(:), invector2(:)
	real(mc_real) outvector(size(invector2,1), size(invector1,1))
        integer(mc_int) i,j 
	do i=1,size(invector1,1)
		do j=1,size(invector2,1)
			outvector(j,i) = invector1(i)*invector2(j)
		end do
	end do
end function mc_outer_product_r

! Extract[argument, selector] -> element
! -------------------------------------------------------------

! vector -> element
function mc_extract_10_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:)
	integer(mc_int), intent(in) :: sel(1)
	integer(mc_int) res

	res = arg(sel(1))
end function mc_extract_10_i

! matrix -> element
function mc_extract_20_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:)
	integer(mc_int), intent(in) :: sel(2)
	integer(mc_int) res

	res = arg(sel(1),sel(2))
end function mc_extract_20_i

! rank 3 tensor -> element
function mc_extract_30_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(3)
	integer(mc_int) res
  
	res = arg(sel(1),sel(2),sel(3))
end function mc_extract_30_i

! rank 4 tensor -> element
function mc_extract_40_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(4)
	integer(mc_int) res

	res = arg(sel(1),sel(2),sel(3),sel(4))
end function mc_extract_40_i

! vector -> element
function mc_extract_10_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:)
	integer(mc_int), intent(in) :: sel(1)
	real(mc_real) res

	res = arg(sel(1))
end function mc_extract_10_r

! matrix -> element
function mc_extract_20_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:)
	integer(mc_int), intent(in) :: sel(2)
	real(mc_real) res

	res = arg(sel(1),sel(2))
end function mc_extract_20_r

! rank 3 tensor -> element
function mc_extract_30_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(3)
	real(mc_real) res
	  
	res = arg(sel(1),sel(2),sel(3))
end function mc_extract_30_r

! rank 4 tensor -> element
function mc_extract_40_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(4)
	real(mc_real) res
	
	res = arg(sel(1),sel(2),sel(3),sel(4))
end function mc_extract_40_r

! Extract[argument, selector] -> vector
! -------------------------------------------------------------

! matrix -> vector
function mc_extract_21_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:)
	integer(mc_int), intent(in) :: sel(1)
	integer(mc_int) res(size(arg,2))

	res = arg(sel(1),:)
end function mc_extract_21_i

! rank 3 tensor -> vector
function mc_extract_31_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(2)
	integer(mc_int) res(size(arg,3))

	res = arg(sel(1),sel(2),:)
end function mc_extract_31_i

! rank 4 tensor -> vector
function mc_extract_41_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(3)
	integer(mc_int) res(size(arg,4))

	res = arg(sel(1),sel(2),sel(3),:)
end function mc_extract_41_i

! matrix -> vector
function mc_extract_21_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:)
	integer(mc_int), intent(in) :: sel(1)
	real(mc_real) res(size(arg,2))

	res = arg(sel(1),:)
end function mc_extract_21_r

! rank 3 tensor -> vector
function mc_extract_31_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(2)
	real(mc_real) res(size(arg,3))

	res = arg(sel(1),sel(2),:)
end function mc_extract_31_r

! rank 4 tensor -> vector
function mc_extract_41_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(3)
	real(mc_real) res(size(arg,4))

	res = arg(sel(1),sel(2),sel(3),:)
end function mc_extract_41_r

! Extract[argument, selector] -> matrix
! -------------------------------------------------------------

! rank 3 tensor -> matrix
function mc_extract_32_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(1)
	integer(mc_int) res(size(arg,2),size(arg,3))

	res = arg(sel(1),:,:)
end function mc_extract_32_i

! rank 4 tensor -> matrix
function mc_extract_42_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(2)
	integer(mc_int) res(size(arg,3),size(arg,4))

	res = arg(sel(1),sel(2),:,:)
end function mc_extract_42_i

! rank 3 tensor -> matrix
function mc_extract_32_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:)
	integer(mc_int), intent(in) :: sel(1)
	real(mc_real) res(size(arg,2),size(arg,3))

	res = arg(sel(1),:,:)
end function mc_extract_32_r

! rank 4 tensor -> matrix
function mc_extract_42_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(2)
	real(mc_real) res(size(arg,3),size(arg,4))

	res = arg(sel(1),sel(2),:,:)
end function mc_extract_42_r

! Extract[argument, selector] -> rank 3 tensor
! -------------------------------------------------------------

! rank 4 tensor -> rank 3 tensor
function mc_extract_43_i(arg, sel) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(1)
	integer(mc_int) res(size(arg,2),size(arg,3),size(arg,4))

	res = arg(sel(1),:,:,:)
end function mc_extract_43_i

! rank 4 tensor -> rank 3 tensor
function mc_extract_43_r(arg, sel) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg(:,:,:,:)
	integer(mc_int), intent(in) :: sel(1)
	real(mc_real) res(size(arg,2),size(arg,3),size(arg,4))

	res = arg(sel(1),:,:,:)
end function mc_extract_43_r


!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 1 arrays of integers 

function mc_equal_1_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:)
	integer(mc_int), intent(in) :: arg2(:)
	logical :: res
	integer :: counter
	if (size(arg1)/=size(arg2)) then
		res = .FALSE.
	else
		res = .TRUE.
		do counter =1, size(arg1), 1
		        if (arg1(counter) /= arg2(counter)) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_1_i


!-------------------------------------------------------------
!The  same as mc_equal_1_i, but called from mc_equal_2_i 
 

function mc_equal_1d_i(arg1, arg2, s) result (res)
	use MathCodePrecision
	integer(mc_int) :: s
	integer(mc_int), intent(in) :: arg1(s)
	integer(mc_int), intent(in) :: arg2(s)
	logical :: res
	integer :: counter
	if (size(arg1)/=size(arg2)) then
		res = .FALSE.
	else
		res = .TRUE.
		do counter =1, size(arg1), 1
		        if (arg1(counter) /= arg2(counter)) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_1d_i

!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 1 arrays of reals 

function mc_equal_1_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:)
	real(mc_real), intent(in) :: arg2(:)
	logical :: res
	integer :: counter
	
	if (size(arg1)/=size(arg2)) then
		res = .FALSE.
	else
		res = .TRUE.
		do counter =1, size(arg1), 1
			if (arg1(counter) /= arg2(counter)) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_1_r

!-------------------------------------------------------------
!The  same as mc_equal_1_r, but called from mc_equal_2_r  

function mc_equal_1d_r(arg1, arg2, s, dummy) result (res)
	use MathCodePrecision
	integer :: s
	real(mc_real), intent(in) :: arg1(s)
	real(mc_real), intent(in) :: arg2(s)
	logical :: res
	integer :: counter
	integer ::dummy
	
	if (size(arg1)/=size(arg2)) then
		res = .FALSE.
	else
		res = .TRUE.
		do counter =1, size(arg1), 1
		        if (arg1(counter) /= arg2(counter)) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_1d_r


!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 2 arrays of integers 

function mc_equal_2_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:)
	integer(mc_int), intent(in) :: arg2(:,:)
	logical :: res
	logical :: tmp
	integer :: counter
	integer :: arrSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrSize = size(arg1,2)		         
		do counter =1, size(arg1,1), 1	
		         tmp = mc_equal_1d_i(arg1(counter,:), arg2(counter,:), arrSize)
		         if (.not.(tmp)) then
				res= .FALSE.
				EXIT
			end if			
		end do
	end if
end function mc_equal_2_i


!-------------------------------------------------------------
!The  same as mc_equal_2_i, but called from mc_equal_3_i 

function mc_equal_2d_i(arg1, arg2, s1, s2) result (res)
	use MathCodePrecision
	integer :: s1
	integer :: s2
	integer(mc_int), intent(in) :: arg1(s1,s2)
	integer(mc_int), intent(in) :: arg2(s1,s2)
	logical :: res
	integer :: counter
	integer :: arrSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrSize = size(arg1,2)		         
		do counter =1, size(arg1,1), 1	
		         if (.not.(mc_equal_1d_i(arg1(counter,:), arg2(counter,:), arrSize))) then
				res= .FALSE.
				EXIT
			end if			
		end do
	end if
end function mc_equal_2d_i


!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 2 arrays of reals 

function mc_equal_2_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:)
	real(mc_real), intent(in) :: arg2(:,:)
	logical :: res
	integer :: counter
	integer :: arrSize
	integer :: dummy
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrSize = size(arg1,2)
		do counter =1, size(arg1,1), 1
		        if (.not.(mc_equal_1d_r(arg1(counter,:), arg2(counter,:), arrSize, dummy))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_2_r

!-------------------------------------------------------------
!The  same as mc_equal_2_r, but called from mc_equal_3_r  

function mc_equal_2d_r(arg1, arg2, s1, s2, dummy) result (res)
	use MathCodePrecision
	integer :: s1
	integer :: s2
	real(mc_real), intent(in) :: arg1(s1,s2)
	real(mc_real), intent(in) :: arg2(s1,s2)
	logical :: res
	integer :: counter
	integer :: arrSize
	integer :: dummy
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrSize = size(arg1,2)
		do counter =1, size(arg1,1), 1
		        if (.not.(mc_equal_1d_r(arg1(counter,:), arg2(counter,:), arrSize, dummy))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_2d_r

!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 3 arrays of integers 

function mc_equal_3_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:,:)
	integer(mc_int), intent(in) :: arg2(:,:,:)
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2)).or. (size(arg1,3)/=size(arg2,3))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		do counter =1, size(arg1,1), 1
		        if (.not.(mc_equal_2d_i(arg1(counter,:,:), arg2(counter,:,:), arrFirstSize, arrSecondSize))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_3_i


!-------------------------------------------------------------
!The  same as mc_equal_3_i, but called from mc_equal_4_i   

function mc_equal_3d_i(arg1, arg2, s1, s2, s3) result (res)
	use MathCodePrecision
	integer :: s1
	integer :: s2
	integer :: s3
	integer(mc_int), intent(in) :: arg1(s1,s2,s3)
	integer(mc_int), intent(in) :: arg2(s1,s2,s3)
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2)).or. (size(arg1,3)/=size(arg2,3))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		do counter =1, size(arg1,1), 1
		        if (.not.(mc_equal_2d_i(arg1(counter,:,:), arg2(counter,:,:), arrFirstSize, arrSecondSize))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_3d_i

!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 3 arrays of reals 

function mc_equal_3_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:,:)
	real(mc_real), intent(in) :: arg2(:,:,:)
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	integer :: dummy
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2)).or. (size(arg1,3)/=size(arg2,3))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		do counter =1, size(arg1,1), 1
		        if (.not.(mc_equal_2d_r(arg1(counter,:,:), arg2(counter,:,:), arrFirstSize, arrSecondSize, dummy))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_3_r

!-------------------------------------------------------------
!The  same as mc_equal_3_r, but called from mc_equal_4_r 


function mc_equal_3d_r(arg1, arg2, s1, s2, s3, dummy) result (res)
	use MathCodePrecision
	integer :: s1
	integer :: s2
	integer :: s3
	real(mc_real), intent(in) :: arg1(s1,s2,s3)
	real(mc_real), intent(in) :: arg2(s1,s2,s3)
	integer :: dummy
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2)).or. (size(arg1,3)/=size(arg2,3))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		do counter =1, size(arg1,1), 1
			if (.not.(mc_equal_2d_r(arg1(counter,:,:), arg2(counter,:,:), arrFirstSize, arrSecondSize, dummy))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_3d_r

!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 4 arrays of integers 

function mc_equal_4_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:,:,:)
	integer(mc_int), intent(in) :: arg2(:,:,:,:)
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	integer :: arrThirdSize
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2) .or.& 
	(size(arg1,3)/=size(arg2,3)) .or. (size(arg1,4)/=size(arg2,4)))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		arrThirdSize= size(arg1,4)
		
		do counter =1, size(arg1,1), 1
			if (.not.(mc_equal_3d_i(arg1(counter,:,:,:), arg2(counter,:,:,:), arrFirstSize, arrSecondSize, arrThirdSize))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_4_i

!-------------------------------------------------------------
! Equal[arg1, arg2] where arg1 and arg2 are rank 4 arrays of reals 

function mc_equal_4_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:,:,:)
	real(mc_real), intent(in) :: arg2(:,:,:,:)
	logical :: res
	integer :: counter
	integer :: arrFirstSize
	integer :: arrSecondSize
	integer :: arrThirdSize
	integer :: dummy
	
	if ((size(arg1,1)/=size(arg2,1)) .or. (size(arg1,2)/=size(arg2,2)).or. &
		(size(arg1,3)/=size(arg2,3)) .or. (size(arg1,4)/=size(arg2,4))) then
		res = .FALSE.
	else
		res = .TRUE.
		arrFirstSize= size(arg1,2)
		arrSecondSize= size(arg1,3)
		arrThirdSize= size(arg1,4)
		do counter =1, size(arg1,1), 1
			if (.not.(mc_equal_3d_r(arg1(counter,:,:,:), arg2(counter,:,:,:), arrFirstSize, arrSecondSize, arrThirdSize,dummy))) then
				res= .FALSE.
				EXIT
			end if
		end do
	end if
end function mc_equal_4_r

!--------------------------------------------------------------------
! Cross[arg1, arg2] where arg1 and arg2 are 1D arrays of integers  

function mc_crossprod_2_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(3)
	integer(mc_int), intent(in) :: arg2(3)
	integer(mc_int) :: res(3)	
	
	res(1) = -(arg1(3) * arg2(2)) + arg1(2)* arg2(3)
	res(2) = arg1(3) * arg2(1) - arg1(1) * arg2(3)
	res(3) = -(arg1(2) * arg2(1)) + arg1(1)*arg2(2)
end function mc_crossprod_2_i


!--------------------------------------------------------------------
! Cross[arg1, arg2] where arg1 and arg2 are 1D arrays of reals 

function mc_crossprod_2_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(3)
	real(mc_real), intent(in) :: arg2(3)
	real(mc_real) :: res(3)	
	
	res(1) = -(arg1(3) * arg2(2)) + arg1(2)* arg2(3)
	res(2) = arg1(3) * arg2(1) - arg1(1) * arg2(3)
	res(3) = -(arg1(2) * arg2(1)) + arg1(1)*arg2(2)
end function mc_crossprod_2_r

!--------------------------------------------------------------------
! Cross[arg1, arg2, arg3 ] where arg1,arg2 and arg3 are 1D arrays of integers 


function mc_crossprod_3_i(arg1, arg2, arg3) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(4)
	integer(mc_int), intent(in) :: arg2(4)
	integer(mc_int), intent(in) :: arg3(4)
	integer(mc_int) :: res(4)	
	
	res(1) = arg1(4)*arg2(3)*arg3(2) - arg1(3)*arg2(4)*arg3(2) - arg1(4)*arg2(2)*arg3(3) &
	+ arg1(2)*arg2(4)*arg3(3) + arg1(3)*arg2(2)*arg3(4) - arg1(2)*arg2(3)*arg3(4)
	res(2) = -(arg1(4)*arg2(3)*arg3(1)) + arg1(3)*arg2(4)*arg3(1) + arg1(4)*arg2(1)*arg3(3) &
	- arg1(1)*arg2(4)*arg3(3) - arg1(3)*arg2(1)*arg3(4) + arg1(1)*arg2(3)*arg3(4)  
	res(3) = arg1(4)*arg2(2)*arg3(1) - arg1(2)*arg2(4)*arg3(1) - arg1(4)*arg2(1)*arg3(2) &
	+ arg1(1)*arg2(4)*arg3(2) + arg1(2)*arg2(1)*arg3(4) - arg1(1)*arg2(2)*arg3(4)
	res(4) = -(arg1(3)*arg2(2)*arg3(1)) + arg1(2)*arg2(3)*arg3(1) + arg1(3)*arg2(1)*arg3(2) &
	- arg1(1)*arg2(3)*arg3(2) - arg1(2)*arg2(1)*arg3(3) + arg1(1)*arg2(2)*arg3(3)
	
end function mc_crossprod_3_i

!--------------------------------------------------------------------
! Cross[arg1, arg2, arg3 ] where arg1,arg2 and arg3 are 1D arrays of reals 


function mc_crossprod_3_r(arg1, arg2, arg3) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(4)
	real(mc_real), intent(in) :: arg2(4)
	real(mc_real), intent(in) :: arg3(4)
	real(mc_real) :: res(4)	
	
	res(1) = arg1(4)*arg2(3)*arg3(2) - arg1(3)*arg2(4)*arg3(2) - arg1(4)*arg2(2)*arg3(3) &
	+ arg1(2)*arg2(4)*arg3(3) + arg1(3)*arg2(2)*arg3(4) - arg1(2)*arg2(3)*arg3(4)
	res(2) = -(arg1(4)*arg2(3)*arg3(1)) + arg1(3)*arg2(4)*arg3(1) + arg1(4)*arg2(1)*arg3(3) &
	- arg1(1)*arg2(4)*arg3(3) - arg1(3)*arg2(1)*arg3(4) + arg1(1)*arg2(3)*arg3(4)  
	res(3) = arg1(4)*arg2(2)*arg3(1) - arg1(2)*arg2(4)*arg3(1) - arg1(4)*arg2(1)*arg3(2) &
	+ arg1(1)*arg2(4)*arg3(2) + arg1(2)*arg2(1)*arg3(4) - arg1(1)*arg2(2)*arg3(4)
	res(4) = -(arg1(3)*arg2(2)*arg3(1)) + arg1(2)*arg2(3)*arg3(1) + arg1(3)*arg2(1)*arg3(2) &
	- arg1(1)*arg2(3)*arg3(2) - arg1(2)*arg2(1)*arg3(3) + arg1(1)*arg2(2)*arg3(3)
	
end function mc_crossprod_3_r

!--------------------------------------------------------------------
! Cross[arg1, arg2, arg3, arg4 ] where arg1,arg2,arg3  and arg4 are 1D arrays of integers 


function mc_crossprod_4_i(arg1, arg2, arg3, arg4) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(5)
	integer(mc_int), intent(in) :: arg2(5)
	integer(mc_int), intent(in) :: arg3(5)
	integer(mc_int), intent(in) :: arg4(5)
	integer(mc_int) :: res(5)	
	
	res(1) = arg1(5)*arg2(4)*arg3(3)*arg4(2) - arg1(4)*arg2(5)*arg3(3)*arg4(2) - arg1(5)*arg2(3)*arg3(4)*arg4(2) &
	+ arg1(3)*arg2(5)*arg3(4)*arg4(2) + arg1(4)*arg2(3)*arg3(5)*arg4(2) - arg1(3)*arg2(4)*arg3(5)*arg4(2) &
	- arg1(5)*arg2(4)*arg3(2)*arg4(3) + arg1(4)*arg2(5)*arg3(2)*arg4(3) + arg1(5)*arg2(2)*arg3(4)*arg4(3) &
	- arg1(2)*arg2(5)*arg3(4)*arg4(3) - arg1(4)*arg2(2)*arg3(5)*arg4(3) + arg1(2)*arg2(4)*arg3(5)*arg4(3) &
	+ arg1(5)*arg2(3)*arg3(2)*arg4(4) - arg1(3)*arg2(5)*arg3(2)*arg4(4) - arg1(5)*arg2(2)*arg3(3)*arg4(4) &
	+ arg1(2)*arg2(5)*arg3(3)*arg4(4) + arg1(3)*arg2(2)*arg3(5)*arg4(4) - arg1(2)*arg2(3)*arg3(5)*arg4(4) &
	- arg1(4)*arg2(3)*arg3(2)*arg4(5) + arg1(3)*arg2(4)*arg3(2)*arg4(5) + arg1(4)*arg2(2)*arg3(3)*arg4(5) &
	- arg1(2)*arg2(4)*arg3(3)*arg4(5) - arg1(3)*arg2(2)*arg3(4)*arg4(5) + arg1(2)*arg2(3)*arg3(4)*arg4(5)
        res(2) = -(arg1(5)*arg2(4)*arg3(3)*arg4(1)) + arg1(4)*arg2(5)*arg3(3)*arg4(1) + arg1(5)*arg2(3)*arg3(4)*arg4(1) &
        - arg1(3)*arg2(5)*arg3(4)*arg4(1) - arg1(4)*arg2(3)*arg3(5)*arg4(1) + arg1(3)*arg2(4)*arg3(5)*arg4(1) &
        + arg1(5)*arg2(4)*arg3(1)*arg4(3) - arg1(4)*arg2(5)*arg3(1)*arg4(3) - arg1(5)*arg2(1)*arg3(4)*arg4(3) &
        + arg1(1)*arg2(5)*arg3(4)*arg4(3) + arg1(4)*arg2(1)*arg3(5)*arg4(3) - arg1(1)*arg2(4)*arg3(5)*arg4(3) &
        - arg1(5)*arg2(3)*arg3(1)*arg4(4) + arg1(3)*arg2(5)*arg3(1)*arg4(4) + arg1(5)*arg2(1)*arg3(3)*arg4(4) &
        - arg1(1)*arg2(5)*arg3(3)*arg4(4) - arg1(3)*arg2(1)*arg3(5)*arg4(4) + arg1(1)*arg2(3)*arg3(5)*arg4(4) &
        + arg1(4)*arg2(3)*arg3(1)*arg4(5) - arg1(3)*arg2(4)*arg3(1)*arg4(5) - arg1(4)*arg2(1)*arg3(3)*arg4(5) &
        + arg1(1)*arg2(4)*arg3(3)*arg4(5) + arg1(3)*arg2(1)*arg3(4)*arg4(5) - arg1(1)*arg2(3)*arg3(4)*arg4(5)
        res(3) = arg1(5)*arg2(4)*arg3(2)*arg4(1) - arg1(4)*arg2(5)*arg3(2)*arg4(1) - arg1(5)*arg2(2)*arg3(4)*arg4(1) &
        + arg1(2)*arg2(5)*arg3(4)*arg4(1) + arg1(4)*arg2(2)*arg3(5)*arg4(1) - arg1(2)*arg2(4)*arg3(5)*arg4(1) &
        - arg1(5)*arg2(4)*arg3(1)*arg4(2) + arg1(4)*arg2(5)*arg3(1)*arg4(2) + arg1(5)*arg2(1)*arg3(4)*arg4(2) &
        - arg1(1)*arg2(5)*arg3(4)*arg4(2) - arg1(4)*arg2(1)*arg3(5)*arg4(2) + arg1(1)*arg2(4)*arg3(5)*arg4(2) &
        + arg1(5)*arg2(2)*arg3(1)*arg4(4) - arg1(2)*arg2(5)*arg3(1)*arg4(4) - arg1(5)*arg2(1)*arg3(2)*arg4(4) &
        + arg1(1)*arg2(5)*arg3(2)*arg4(4) + arg1(2)*arg2(1)*arg3(5)*arg4(4) - arg1(1)*arg2(2)*arg3(5)*arg4(4) &
        - arg1(4)*arg2(2)*arg3(1)*arg4(5) + arg1(2)*arg2(4)*arg3(1)*arg4(5) + arg1(4)*arg2(1)*arg3(2)*arg4(5) &
        - arg1(1)*arg2(4)*arg3(2)*arg4(5) - arg1(2)*arg2(1)*arg3(4)*arg4(5) + arg1(1)*arg2(2)*arg3(4)*arg4(5)
        res(4) = -(arg1(5)*arg2(3)*arg3(2)*arg4(1)) + arg1(3)*arg2(5)*arg3(2)*arg4(1) + arg1(5)*arg2(2)*arg3(3)*arg4(1)& 
        - arg1(2)*arg2(5)*arg3(3)*arg4(1) - arg1(3)*arg2(2)*arg3(5)*arg4(1) + arg1(2)*arg2(3)*arg3(5)*arg4(1) &
        + arg1(5)*arg2(3)*arg3(1)*arg4(2) - arg1(3)*arg2(5)*arg3(1)*arg4(2) - arg1(5)*arg2(1)*arg3(3)*arg4(2) &
        + arg1(1)*arg2(5)*arg3(3)*arg4(2) + arg1(3)*arg2(1)*arg3(5)*arg4(2) - arg1(1)*arg2(3)*arg3(5)*arg4(2) &
        - arg1(5)*arg2(2)*arg3(1)*arg4(3) + arg1(2)*arg2(5)*arg3(1)*arg4(3) + arg1(5)*arg2(1)*arg3(2)*arg4(3) &
        - arg1(1)*arg2(5)*arg3(2)*arg4(3) - arg1(2)*arg2(1)*arg3(5)*arg4(3) + arg1(1)*arg2(2)*arg3(5)*arg4(3) &
        + arg1(3)*arg2(2)*arg3(1)*arg4(5) - arg1(2)*arg2(3)*arg3(1)*arg4(5) - arg1(3)*arg2(1)*arg3(2)*arg4(5) &
        + arg1(1)*arg2(3)*arg3(2)*arg4(5) + arg1(2)*arg2(1)*arg3(3)*arg4(5) - arg1(1)*arg2(2)*arg3(3)*arg4(5)
        res(5) = arg1(4)*arg2(3)*arg3(2)*arg4(1) - arg1(3)*arg2(4)*arg3(2)*arg4(1) - arg1(4)*arg2(2)*arg3(3)*arg4(1) &
        + arg1(2)*arg2(4)*arg3(3)*arg4(1) + arg1(3)*arg2(2)*arg3(4)*arg4(1) - arg1(2)*arg2(3)*arg3(4)*arg4(1) &
        - arg1(4)*arg2(3)*arg3(1)*arg4(2) + arg1(3)*arg2(4)*arg3(1)*arg4(2) + arg1(4)*arg2(1)*arg3(3)*arg4(2) &
        -arg1(1)*arg2(4)*arg3(3)*arg4(2) - arg1(3)*arg2(1)*arg3(4)*arg4(2) + arg1(1)*arg2(3)*arg3(4)*arg4(2) &
        + arg1(4)*arg2(2)*arg3(1)*arg4(3) - arg1(2)*arg2(4)*arg3(1)*arg4(3) - arg1(4)*arg2(1)*arg3(2)*arg4(3) &
        + arg1(1)*arg2(4)*arg3(2)*arg4(3) + arg1(2)*arg2(1)*arg3(4)*arg4(3) - arg1(1)*arg2(2)*arg3(4)*arg4(3) &
        - arg1(3)*arg2(2)*arg3(1)*arg4(4) + arg1(2)*arg2(3)*arg3(1)*arg4(4) + arg1(3)*arg2(1)*arg3(2)*arg4(4) &
        - arg1(1)*arg2(3)*arg3(2)*arg4(4) - arg1(2)*arg2(1)*arg3(3)*arg4(4) + arg1(1)*arg2(2)*arg3(3)*arg4(4)
        
end function mc_crossprod_4_i


!--------------------------------------------------------------------
! Cross[arg1, arg2, arg3, arg4 ] where arg1,arg2, arg3 and arg4 are 1D arrays of reals 


function mc_crossprod_4_r(arg1, arg2, arg3, arg4) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(5)
	real(mc_real), intent(in) :: arg2(5)
	real(mc_real), intent(in) :: arg3(5)
	real(mc_real), intent(in) :: arg4(5)
	real(mc_real) :: res(5)	

	res(1) = arg1(5)*arg2(4)*arg3(3)*arg4(2) - arg1(4)*arg2(5)*arg3(3)*arg4(2) - arg1(5)*arg2(3)*arg3(4)*arg4(2) &
	+ arg1(3)*arg2(5)*arg3(4)*arg4(2) + arg1(4)*arg2(3)*arg3(5)*arg4(2) - arg1(3)*arg2(4)*arg3(5)*arg4(2) &
	- arg1(5)*arg2(4)*arg3(2)*arg4(3) + arg1(4)*arg2(5)*arg3(2)*arg4(3) + arg1(5)*arg2(2)*arg3(4)*arg4(3) &
	- arg1(2)*arg2(5)*arg3(4)*arg4(3) - arg1(4)*arg2(2)*arg3(5)*arg4(3) + arg1(2)*arg2(4)*arg3(5)*arg4(3) &
	+ arg1(5)*arg2(3)*arg3(2)*arg4(4) - arg1(3)*arg2(5)*arg3(2)*arg4(4) - arg1(5)*arg2(2)*arg3(3)*arg4(4) &
	+ arg1(2)*arg2(5)*arg3(3)*arg4(4) + arg1(3)*arg2(2)*arg3(5)*arg4(4) - arg1(2)*arg2(3)*arg3(5)*arg4(4) &
	- arg1(4)*arg2(3)*arg3(2)*arg4(5) + arg1(3)*arg2(4)*arg3(2)*arg4(5) + arg1(4)*arg2(2)*arg3(3)*arg4(5) &
	- arg1(2)*arg2(4)*arg3(3)*arg4(5) - arg1(3)*arg2(2)*arg3(4)*arg4(5) + arg1(2)*arg2(3)*arg3(4)*arg4(5)
        res(2) = -(arg1(5)*arg2(4)*arg3(3)*arg4(1)) + arg1(4)*arg2(5)*arg3(3)*arg4(1) + arg1(5)*arg2(3)*arg3(4)*arg4(1) &
        - arg1(3)*arg2(5)*arg3(4)*arg4(1) - arg1(4)*arg2(3)*arg3(5)*arg4(1) + arg1(3)*arg2(4)*arg3(5)*arg4(1) &
        + arg1(5)*arg2(4)*arg3(1)*arg4(3) - arg1(4)*arg2(5)*arg3(1)*arg4(3) - arg1(5)*arg2(1)*arg3(4)*arg4(3) &
        + arg1(1)*arg2(5)*arg3(4)*arg4(3) + arg1(4)*arg2(1)*arg3(5)*arg4(3) - arg1(1)*arg2(4)*arg3(5)*arg4(3) &
        - arg1(5)*arg2(3)*arg3(1)*arg4(4) + arg1(3)*arg2(5)*arg3(1)*arg4(4) + arg1(5)*arg2(1)*arg3(3)*arg4(4) &
        - arg1(1)*arg2(5)*arg3(3)*arg4(4) - arg1(3)*arg2(1)*arg3(5)*arg4(4) + arg1(1)*arg2(3)*arg3(5)*arg4(4) &
        + arg1(4)*arg2(3)*arg3(1)*arg4(5) - arg1(3)*arg2(4)*arg3(1)*arg4(5) - arg1(4)*arg2(1)*arg3(3)*arg4(5) &
        + arg1(1)*arg2(4)*arg3(3)*arg4(5) + arg1(3)*arg2(1)*arg3(4)*arg4(5) - arg1(1)*arg2(3)*arg3(4)*arg4(5)
        res(3) = arg1(5)*arg2(4)*arg3(2)*arg4(1) - arg1(4)*arg2(5)*arg3(2)*arg4(1) - arg1(5)*arg2(2)*arg3(4)*arg4(1) &
        + arg1(2)*arg2(5)*arg3(4)*arg4(1) + arg1(4)*arg2(2)*arg3(5)*arg4(1) - arg1(2)*arg2(4)*arg3(5)*arg4(1) &
        - arg1(5)*arg2(4)*arg3(1)*arg4(2) + arg1(4)*arg2(5)*arg3(1)*arg4(2) + arg1(5)*arg2(1)*arg3(4)*arg4(2) &
        - arg1(1)*arg2(5)*arg3(4)*arg4(2) - arg1(4)*arg2(1)*arg3(5)*arg4(2) + arg1(1)*arg2(4)*arg3(5)*arg4(2) &
        + arg1(5)*arg2(2)*arg3(1)*arg4(4) - arg1(2)*arg2(5)*arg3(1)*arg4(4) - arg1(5)*arg2(1)*arg3(2)*arg4(4) &
        + arg1(1)*arg2(5)*arg3(2)*arg4(4) + arg1(2)*arg2(1)*arg3(5)*arg4(4) - arg1(1)*arg2(2)*arg3(5)*arg4(4) &
        - arg1(4)*arg2(2)*arg3(1)*arg4(5) + arg1(2)*arg2(4)*arg3(1)*arg4(5) + arg1(4)*arg2(1)*arg3(2)*arg4(5) &
        - arg1(1)*arg2(4)*arg3(2)*arg4(5) - arg1(2)*arg2(1)*arg3(4)*arg4(5) + arg1(1)*arg2(2)*arg3(4)*arg4(5)
        res(4) = -(arg1(5)*arg2(3)*arg3(2)*arg4(1)) + arg1(3)*arg2(5)*arg3(2)*arg4(1) + arg1(5)*arg2(2)*arg3(3)*arg4(1)& 
        - arg1(2)*arg2(5)*arg3(3)*arg4(1) - arg1(3)*arg2(2)*arg3(5)*arg4(1) + arg1(2)*arg2(3)*arg3(5)*arg4(1) &
        + arg1(5)*arg2(3)*arg3(1)*arg4(2) - arg1(3)*arg2(5)*arg3(1)*arg4(2) - arg1(5)*arg2(1)*arg3(3)*arg4(2) &
        + arg1(1)*arg2(5)*arg3(3)*arg4(2) + arg1(3)*arg2(1)*arg3(5)*arg4(2) - arg1(1)*arg2(3)*arg3(5)*arg4(2) &
        - arg1(5)*arg2(2)*arg3(1)*arg4(3) + arg1(2)*arg2(5)*arg3(1)*arg4(3) + arg1(5)*arg2(1)*arg3(2)*arg4(3) &
        - arg1(1)*arg2(5)*arg3(2)*arg4(3) - arg1(2)*arg2(1)*arg3(5)*arg4(3) + arg1(1)*arg2(2)*arg3(5)*arg4(3) &
        + arg1(3)*arg2(2)*arg3(1)*arg4(5) - arg1(2)*arg2(3)*arg3(1)*arg4(5) - arg1(3)*arg2(1)*arg3(2)*arg4(5) &
        + arg1(1)*arg2(3)*arg3(2)*arg4(5) + arg1(2)*arg2(1)*arg3(3)*arg4(5) - arg1(1)*arg2(2)*arg3(3)*arg4(5)
        res(5) = arg1(4)*arg2(3)*arg3(2)*arg4(1) - arg1(3)*arg2(4)*arg3(2)*arg4(1) - arg1(4)*arg2(2)*arg3(3)*arg4(1) &
        + arg1(2)*arg2(4)*arg3(3)*arg4(1) + arg1(3)*arg2(2)*arg3(4)*arg4(1) - arg1(2)*arg2(3)*arg3(4)*arg4(1) &
        - arg1(4)*arg2(3)*arg3(1)*arg4(2) + arg1(3)*arg2(4)*arg3(1)*arg4(2) + arg1(4)*arg2(1)*arg3(3)*arg4(2) &
        -arg1(1)*arg2(4)*arg3(3)*arg4(2) - arg1(3)*arg2(1)*arg3(4)*arg4(2) + arg1(1)*arg2(3)*arg3(4)*arg4(2) &
        + arg1(4)*arg2(2)*arg3(1)*arg4(3) - arg1(2)*arg2(4)*arg3(1)*arg4(3) - arg1(4)*arg2(1)*arg3(2)*arg4(3) &
        + arg1(1)*arg2(4)*arg3(2)*arg4(3) + arg1(2)*arg2(1)*arg3(4)*arg4(3) - arg1(1)*arg2(2)*arg3(4)*arg4(3) &
        - arg1(3)*arg2(2)*arg3(1)*arg4(4) + arg1(2)*arg2(3)*arg3(1)*arg4(4) + arg1(3)*arg2(1)*arg3(2)*arg4(4) &
        - arg1(1)*arg2(3)*arg3(2)*arg4(4) - arg1(2)*arg2(1)*arg3(3)*arg4(4) + arg1(1)*arg2(2)*arg3(3)*arg4(4)
	
       
end function mc_crossprod_4_r


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes an integer and returns array of one integer  

function mc_mkc_1_i(arg1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1
	integer(mc_int) :: res(1)
	
	res(1) = arg1
	
end function mc_mkc_1_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a real and returns array of one real 

function mc_mkc_1_r(arg1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1
	real(mc_real) :: res(1)
	
	res(1) = arg1
end function mc_mkc_1_r


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 1D array of integers and returns 2D array of integers 

function mc_mkc_2_i(arg1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:)
	integer(mc_int) :: res(size(arg1),1)
        integer I	
	forall(I=1:size(arg1))
		res(I,1) = arg1(I)
	end forall
	
end function mc_mkc_2_i


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 1D array of reals and returns 2D array of reals 

function mc_mkc_2_r(arg1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:)
	real(mc_real) :: res(size(arg1,1),1)
	integer I
	forall(I=1:size(arg1))
		res(I,1) = arg1(I)
	end forall
	
end function mc_mkc_2_r

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 2D array of integers and returns 3D array of integers 

function mc_mkc_3_i(arg1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:)
	integer(mc_int) :: res(size(arg1,1),size(arg1,2),1)
	
	integer I,J
	forall(I=1:size(arg1,1),J=1:size(arg1,2))
		res(I,J,1) = arg1(I,J)
	end forall
	
end function mc_mkc_3_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 2D array of reals and returns 3D array of reals 

function mc_mkc_3_r(arg1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:)
	real(mc_real) :: res(size(arg1,1),size(arg1,2),1)
	integer I,J
	forall(I=1:size(arg1,1),J=1:size(arg1,2))
		res(I,J,1) = arg1(I,J)
	end forall
	
end function mc_mkc_3_r

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 3D array of integers and returns 4D array of integers 

function mc_mkc_4_i(arg1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:,:)
	integer(mc_int) :: res(size(arg1,1),size(arg1,2),size(arg1,3),1)
	integer I,J,K
	forall(I=1:size(arg1,1),J=1:size(arg1,2),K=1:size(arg1,3))
		res(I,J,K,1) = arg1(I,J,K)
	end forall
	
end function mc_mkc_4_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 3D array of reals and returns 4D array of reals 

function mc_mkc_4_r(arg1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:,:)
	real(mc_real) :: res(size(arg1,1),size(arg1,2),size(arg1,3),1)
	integer I,J,K
	forall(I=1:size(arg1,1),J=1:size(arg1,2),K=1:size(arg1,3))
		res(I,J,K,1) = arg1(I,J,K)
	end forall
	
end function mc_mkc_4_r


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 1D array of integer and returns 1D array of size size(arg2) + 1  

function mc_mkpc_1_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1
	integer(mc_int), intent(in) :: arg2(:)
	integer(mc_int) :: res(size(arg2,1)+1)
	integer :: ind
	
	res(1) = arg1
	do ind=2, size(res, 1), 1
		res(ind) = arg2(ind-1)
	end do
end function mc_mkpc_1_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 1D array of real and returns 1D array of size size(arg2) + 1  

function mc_mkpc_1_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1
	real(mc_real), intent(in) :: arg2(:)
	real(mc_real) :: res(size(arg2,1)+1)
	integer :: ind
	
	res(1) = arg1
	do ind=2, size(res, 1), 1
		res(ind) = arg2(ind-1)
	end do
	
end function mc_mkpc_1_r

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 2D array of integers and returns array of larger size

function mc_mkpc_2_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:)
	integer(mc_int), intent(in) :: arg2(:,:)
	integer(mc_int) :: res(size(arg2,1),(size(arg2,2)+1))
	integer I,J		
	forall(I=1:size(arg1,1))
		res(I,1) = arg1(I)
	end forall
	
	forall(I=1:size(arg2,1), J=2:size(arg2,2)+1)
		res(I,J) = arg2(I,J-1)
	end forall
	
	
end function mc_mkpc_2_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 2D array of reals and returns array of larger size

function mc_mkpc_2_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:)
	real(mc_real), intent(in) :: arg2(:,:)
	real(mc_real) :: res(size(arg2,1),(size(arg2,2)+1))	
	integer I,J	
	forall(I=1:size(arg1,1))
		res(I,1) = arg1(I)
	end forall
	
	forall(I=1:size(arg2,1), J=2:size(arg2,2)+1)
		res(I,J) = arg2(I,J-1)
	end forall
	
end function mc_mkpc_2_r


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 3D array of integers and returns array of larger size

function mc_mkpc_3_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:)
	integer(mc_int), intent(in) :: arg2(:,:,:)
	integer(mc_int) :: res(size(arg2,1),size(arg2,2),(size(arg2,3)+1))
	integer I,J,K
	forall(I=1:size(arg1,1),J=1:size(arg1,2))
		res(I,J,1) = arg1(I,J)
	end forall
	
	forall(I=1:size(arg2,1), J=1:size(arg2,2),K=2:size(arg2,3)+1)
		res(I,J,K) = arg2(I,J,K-1)
	end forall
	
end function mc_mkpc_3_i


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 3D array of reals and returns array of larger size

function mc_mkpc_3_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:)
	real(mc_real), intent(in) :: arg2(:,:,:)
	real(mc_real) :: res(size(arg2,1),size(arg2,2),(size(arg2,3)+1))
	integer I,J,K
	forall(I=1:size(arg1,1),J=1:size(arg1,2))
		res(I,J,1) = arg1(I,J)
	end forall
	
	forall(I=1:size(arg2,1), J=1:size(arg2,2),K=2:size(arg2,3)+1)
		res(I,J,K) = arg2(I,J,K-1)
	end forall
	
end function mc_mkpc_3_r


!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 4D array of integers and returns array of larger size

function mc_mkpc_4_i(arg1, arg2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arg1(:,:,:)
	integer(mc_int), intent(in) :: arg2(:,:,:,:)
	integer(mc_int) :: res(size(arg2,1),size(arg2,2),size(arg2,3),(size(arg2,4)+1))
	integer I,J,K,L
	forall(I=1:size(arg1,1),J=1:size(arg1,2),K=1:size(arg1,3))
		res(I,J,K,1) = arg1(I,J,K)
	end forall
	
	forall(I=1:size(arg2,1), J=1:size(arg2,2),K=1:size(arg2,3),L=2:size(arg2,4)+1)
		res(I,J,K,L) = arg2(I,J,K,L-1)
	end forall
	
end function mc_mkpc_4_i

!--------------------------------------------------------------------------------
! Used for processing of List[]. Takes a 4D array of reals and returns array of larger size

function mc_mkpc_4_r(arg1, arg2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arg1(:,:,:)
	real(mc_real), intent(in) :: arg2(:,:,:,:)
	real(mc_real) :: res(size(arg2,1),size(arg2,2),size(arg2,3),(size(arg2,4)+1))
	integer I,J,K,L
	forall(I=1:size(arg1,1),J=1:size(arg1,2),K=1:size(arg1,3))
		res(I,J,K,1) = arg1(I,J,K)
	end forall
	
	forall(I=1:size(arg2,1), J=1:size(arg2,2),K=1:size(arg2,3),L=2:size(arg2,4)+1)
		res(I,J,K,L) = arg2(I,J,K,L-1)
	end forall
	
end function mc_mkpc_4_r

!--------------------------------------------------------------------------------
! Part[List[],n] for integers, where List[] is 1D array

function mc_getpart1_1_i(arr, ind1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int) :: res
	
	res = arr(ind1)
	
end function mc_getpart1_1_i

!--------------------------------------------------------------------------------
! Part[List[],n] for reals , where List[] is 1D array

function mc_getpart1_1_r(arr, ind1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:)
	integer(mc_int), intent(in) :: ind1
	real(mc_int) :: res
	
	res = arr(ind1)
	
end function mc_getpart1_1_r

!--------------------------------------------------------------------------------
! Part[List[],n] for integers, where List[] is 2D array

function mc_getpart1_2_i(arr, ind1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int) :: res (size(arr, 1))
	
	res = arr(:, ind1) 
end function mc_getpart1_2_i

!--------------------------------------------------------------------------------
! Part[List[],n] for integers, where List[] is 2D array

function mc_getpart1_2_r(arr, ind1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:)
	integer(mc_int), intent(in) :: ind1
	real(mc_real) :: res (size(arr, 1))
	
	res = arr(:,ind1) 
end function mc_getpart1_2_r


!--------------------------------------------------------------------------------
! Part[List[],n] for integers, where List[] is 3D array

function mc_getpart1_3_i(arr, ind1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int) :: res (size(arr,1),size(arr,2))
	
	res = arr(:,:,ind1)
end function mc_getpart1_3_i

!--------------------------------------------------------------------------------
! Part[List[],n] for reals, where List[] is 3D array

function mc_getpart1_3_r(arr, ind1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	real(mc_real) :: res (size(arr,1),size(arr,2))
	
	res = arr(:,:,ind1)
end function mc_getpart1_3_r

!--------------------------------------------------------------------------------
! Part[List[],n] for integers, where List[] is 4D array

function mc_getpart1_4_i(arr, ind1) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int) :: res (size(arr,1),size(arr,2),size(arr,3))
	
	res= arr(:, : , :,ind1)
end function mc_getpart1_4_i

!--------------------------------------------------------------------------------
! Part[List[],n] for reals, where List[] is 4D array

function mc_getpart1_4_r(arr, ind1) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	real(mc_real) :: res (size(arr,1),size(arr,2),size(arr,3))
	
	res= arr(:, : , :,ind1)
end function mc_getpart1_4_r

!--------------------------------------------------------------------------------
! Part[List[],n, n] for integers, where List[] is 2D array

function mc_getpart2_2_i(arr, ind1,ind2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int) :: res 
	
	res = arr(ind1, ind2)
	
end function mc_getpart2_2_i

!--------------------------------------------------------------------------------
! Part[List[],n, n] for reals, where List[] is 2D array

function mc_getpart2_2_r(arr, ind1,ind2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	real(mc_real) :: res 
	
	res = arr(ind1, ind2)
	
end function mc_getpart2_2_r

!--------------------------------------------------------------------------------
! Part[List[],n, n] for integers, where List[] is 3D array

function mc_getpart2_3_i(arr, ind1,ind2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int) :: res (size(arr,1))
	
	res= arr(:,ind1, ind2)
	
end function mc_getpart2_3_i

!--------------------------------------------------------------------------------
! Part[List[],n, n] for reals, where List[] is 3D array

function mc_getpart2_3_r(arr, ind1,ind2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	real(mc_real) :: res (size(arr,1))
	
	res= arr(:,ind1, ind2)
	
end function mc_getpart2_3_r


!--------------------------------------------------------------------------------
! Part[List[],n, n] for integers, where List[] is 4D array

function mc_getpart2_4_i(arr, ind1,ind2) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int) :: res (size(arr,1),size(arr,2)) 
	
	res= arr(:,:,ind1,ind2)
end function mc_getpart2_4_i

!--------------------------------------------------------------------------------
! Part[List[],n, n] for reals, where List[] is 4D array

function mc_getpart2_4_r(arr, ind1,ind2) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	real(mc_real) :: res (size(arr,1),size(arr,2)) 
	
	res= arr(:,:,ind1,ind2)
end function mc_getpart2_4_r

!--------------------------------------------------------------------------------
! Part[List[],n, n, n] for integers, where List[] is 3D array

function mc_getpart3_3_i(arr, ind1,ind2, ind3) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	integer(mc_int) :: res
	
	res = arr(ind1, ind2, ind3)
end function mc_getpart3_3_i


!--------------------------------------------------------------------------------
! Part[List[],n, n, n] for reals, where List[] is 3D array

function mc_getpart3_3_r(arr, ind1,ind2, ind3) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	real(mc_real) :: res
	
	res = arr(ind1, ind2, ind3)
end function mc_getpart3_3_r


!--------------------------------------------------------------------------------
! Part[List[],n, n, n] for integers, where List[] is 4D array

function mc_getpart3_4_i(arr, ind1,ind2, ind3) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	integer(mc_int) :: res(size(arr,1))
	
	res = arr(:,ind1, ind2, ind3)
end function mc_getpart3_4_i

!--------------------------------------------------------------------------------
! Part[List[],n, n, n] for reals, where List[] is 4D array

function mc_getpart3_4_r(arr, ind1,ind2, ind3) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	real(mc_real) :: res(size(arr,1))
	
	res = arr(:,ind1, ind2, ind3)
end function mc_getpart3_4_r

!--------------------------------------------------------------------------------
! Part[List[],n, n, n, n] for integers, where List[] is 4D array

function mc_getpart4_4_i(arr, ind1,ind2, ind3, ind4) result (res)
	use MathCodePrecision
	integer(mc_int), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	integer(mc_int), intent(in) :: ind4
	integer(mc_int) :: res
	
	res= arr(ind1, ind2, ind3, ind4)
end function mc_getpart4_4_i

!--------------------------------------------------------------------------------
! Part[List[],n, n, n, n] for reals, where List[] is 4D array

function mc_getpart4_4_r(arr, ind1,ind2, ind3, ind4) result (res)
	use MathCodePrecision
	real(mc_real), intent(in) :: arr(:,:,:,:)
	integer(mc_int), intent(in) :: ind1
	integer(mc_int), intent(in) :: ind2
	integer(mc_int), intent(in) :: ind3
	integer(mc_int), intent(in) :: ind4
	real(mc_real) :: res
	
	res= arr(ind1, ind2, ind3, ind4)
end function mc_getpart4_4_r


end module MathCodeSheep



